% MAIN FILE (ROSA-MC)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%   Reaction-Oriented Sensitivity Analysis  %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%             MC COMPUTATION                %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%         Authors: Zhang/Goutsias           %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%        Last Modified: 09/21/2010          %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;      % clear all parameters
close all;      % close all files

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Input parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

rlambda = 0.4;   % size of biochemical fluctuations

L       = 10000; % number of Latin hypercube samples
theta   = 0.1;   % sensitivity threshold
time    = 240;   % simulation time (sec)

iode    = 1;    % iode=0 use of Matlab ODE solver (slow)
                % iode=1 use of C++ ODE solver (fast)

parameters_EGFR; % system parameters
x0_vec_EGFR;     % initial molecular numbers

%
% Perturbed nominal values
%

load kappaNew.mat;
kappa = kappan;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

durtol  = 0.05;                                     % duration tolerance
tresol  = 0.2;                                      % time resolution (in sec)
tpoints = (0:tresol:time);                          % Time points for ODE integration
odetol  = 1e-6;                                     % tolerance for ODE solver
options = odeset('RelTol',odetol,'AbsTol',odetol);  % options for Matlab ODE solver

tmax    = size(tpoints,2);
x       = repmat(0,[tmax,N]); 
reac(1:M) = 1:M; 

ndata   = time/tresol;
timings = [0,time,ndata];

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Estimate variances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

varest;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate SIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

senseit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Write SI results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

writeit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Save workspace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

save('..\results\workspace\rosa.mat');