% Variance-based sensitivity indices (ROSA-MC)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%   Reaction-Oriented Sensitivity Analysis  %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%             MC COMPUTATION                %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%         Authors: Zhang/Goutsias           %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%        Last Modified: 09/14/2010          %%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% First-order calculations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% Calculate SESIs

sigmat = varfTT./varmTT;
sigmad = varfDD./varmDD;
sigmas = varfSS./varmSS;
sigmar = varfRR./varmRR;

% Calculate TESIs 

taut = 1 - varvTT./varmTT;
taud = 1 - varvDD./varmDD;
taus = 1 - varvSS./varmSS;
taur = 1 - varvRR./varmRR;

% Calculate JESIs

etat = taut - sigmat;
etad = taud - sigmad;
etas = taus - sigmas;
etar = taur - sigmar;

% Calculate delta

deltat = 1 - sum(sigmat);
deltad = 1 - sum(sigmad);
deltas = 1 - sum(sigmas);
deltar = 1 - sum(sigmar);

deltat = deltat*(deltat > 0);
deltad = deltad*(deltad > 0);
deltas = deltas*(deltas > 0);
deltar = deltar*(deltar > 0);

% Sort SESI values

[ssigmat,IT] = sort(sigmat);
[ssigmad,ID] = sort(sigmad);
[ssigmas,IS] = sort(sigmas);
[ssigmar,IR] = sort(sigmar);

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Second-order calculations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% Calculate DESIs

sigma2t = var2fTT./varmmTT;
sigma2d = var2fDD./varmmDD;
sigma2s = var2fSS./varmmSS;
sigma2r = var2fRR./varmmRR;

% Sort DESI values

for m = 1:M
    for n = 1:m
        sigma2t(m,n) = 0;  % set diagonal and lower left side of matrix to
        sigma2d(m,n) = 0;  % zero to avoid redundancy in sorting due 
        sigma2s(m,n) = 0;  % to symmetry
        sigma2r(m,n) = 0;
    end
end
        
for m = 1:M,
   vsigma2t((m-1)*M+1:m*M) = sigma2t(m,:);
   vsigma2d((m-1)*M+1:m*M) = sigma2d(m,:);
   vsigma2s((m-1)*M+1:m*M) = sigma2s(m,:);
   vsigma2r((m-1)*M+1:m*M) = sigma2r(m,:);
end

[svsigma2t,I2T] = sort(vsigma2t);
[svsigma2d,I2D] = sort(vsigma2d);
[svsigma2s,I2S] = sort(vsigma2s);
[svsigma2r,I2R] = sort(vsigma2r);

% Calculate PESIs

for m = 1:M
    nut(m,1:M) = sigma2t(m,1:M) - sigmat(m) - sigmat(1:M);
    nud(m,1:M) = sigma2d(m,1:M) - sigmad(m) - sigmad(1:M);
    nus(m,1:M) = sigma2s(m,1:M) - sigmas(m) - sigmas(1:M);
    nur(m,1:M) = sigma2r(m,1:M) - sigmar(m) - sigmar(1:M);
end

nut = nut.*(nut > 0);
nud = nud.*(nud > 0);
nus = nus.*(nus > 0);
nur = nur.*(nur > 0);

% Calculate gamma

gammat = 1 - sum(sigmat) - sum(sum(nut));
gammad = 1 - sum(sigmad) - sum(sum(nud));
gammas = 1 - sum(sigmas) - sum(sum(nus));
gammar = 1 - sum(sigmar) - sum(sum(nur));

gammat = gammat*(gammat > 0);
gammad = gammad*(gammad > 0);
gammas = gammas*(gammas > 0);
gammar = gammar*(gammar > 0);