% MAIN FILE (NROSA-DA)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%  Noise-Reduced Reaction-Oriented Sensitivity Analysis  %%%%%%%%%
%%%%%%%%%%            Derivative-Based Approximation              %%%%%%%%%
%%%%%%%%%%               Authors: Zhang/Goutsias                  %%%%%%%%%
%%%%%%%%%%              Last Modified: 09/17/2010                 %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;          % clear all parameters
close all;          % close all files

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Input parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

L     = 1000;       % number of Latin hypercube samples
delta = 0.01;       % size of finite-difference
sigma = 1.0;        % size of experimental perturbations
theta = 0.1;        % sensitivity threshold
time  = 240;        % simulation time (sec)

iode    = 1;        % iode=0 use of Matlab ODE solver (slow)
                    % iode=1 use of C++ ODE solver (fast)

parameters_EGFR;    % system parameters
x0_vec_EGFR;        % initial molecular numbers

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

durtol  = 0.05;                           % duration tolerance
tresol  = 0.2;                            % time resolution (in sec)
tpoints = (0:tresol:time);                % time points for ODE integration
odetol  = 1e-6;                           % tolerance for ODE solver
options = odeset('RelTol',odetol,'AbsTol',odetol);  % options for Matlab ODE solver

tmax    = size(tpoints,2);

A       = zeros((2*M),(M+N));
vstoich = (-stoich.*(stoich < 0))';

% Michaelis-Menten kinetics

vstoich(7,5)   =  0;
vstoich(8,4)   =  0;
vstoich(8,5)   = -1;
vstoich(15,9)  =  0;
vstoich(16,6)  =  0;
vstoich(16,9)  = -1;
vstoich(31,18) =  0;
vstoich(32,15) =  0;
vstoich(32,18) = -1;
A(:,(M+1):(M+N)) = vstoich;

for m = 1:M
    if (m == 4) || (m == 8) || (m == 16) % Michaelis-Menten kinetics
        A((2*m-1),m) = -1;
    else
        A((2*m-1),m) = -1;
        A((2*m),m) = -1;
    end
end

load V.mat
 
P = size(V,2);     % number of experimental factors

reac(1:M) = 1:M;

ndata   = time/tresol;
timings = [0,time,ndata];
    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate SIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

senseit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Write sensitivity results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

writeit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Save workspace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

save('..\results\workspace\nrosa_dv.mat');