% MAIN FILE (NROSA-MC)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Noise Reduced Reaction-Oriented Sensitivity Analysis %%%%%%%%%%
%%%%%%%%%%%    MC COMPUTATION WITH DIMENSIONALITY REDUCTION      %%%%%%%%%%
%%%%%%%%%%%              Authors: Zhang/Goutsias                 %%%%%%%%%%
%%%%%%%%%%%             Last Modified: 09/18/2010                %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;       % clear all parameters
close all;       % close all files

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Input parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

L       = 10000; % number of Latin hypercube samples
theta   = 0.1;   % sensitivity threshold
rlambda = 0.4;   % size of biochemical fluctuations
sigma   = 1.0;   % size of experimental fluctuations 
time    = 240;   % simulation time (sec)

iode    = 1;     % iode=0 use of Matlab ODE solver (slow)
                 % iode=1 use of custom ODE solver (fast)
                
parameters_EGFR; % system parameters
x0_vec_EGFR;     % initial molecular numbers

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initialize
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

durtol  = 0.05;                           % duration tolerance
tresol  = 0.2;                            % time resolution (in min)
tpoints = (0:tresol:time);                % Time points for ODE integration
odetol  = 1e-6;                           % tolerance for ODE solver
options = odeset('RelTol',1e-6,'AbsTol',1e-6);  % tolerances for ODE solver
tmax    = size(tpoints,2);

A       = zeros((2*M),(M+N));
vstoich = (-stoich.*(stoich < 0))';

reac = [0, 3, 4, 7, 9, 15, 16]; % OHA-detected influential reactions
nrea = [1, 2, 5, 6, 8, 10, 11, 12, 13, 14, 17, 18, 19, 20, 21, 22, 23, 24, 25]; % OHA-detected non-influential reactions

% Michaelis-Menten kinetics

vstoich(7,5)   =  0;
vstoich(8,4)   =  0;
vstoich(8,5)   = -1;
vstoich(15,9)  =  0;
vstoich(16,6)  =  0;
vstoich(16,9)  = -1;
vstoich(31,18) =  0;
vstoich(32,15) =  0;
vstoich(32,18) = -1;

A(:,(M+1):(M+N)) = vstoich;
for m = 1:M
    if (m == 4) || (m == 8) || (m == 16)  % Michaelis-Menten kinetics
        A((2*m-1),m) = -1;
    else
        A((2*m-1),m) = -1;
        A((2*m),m) = -1;
    end
end

load V.mat
 
P = size(V,2);     % number of noise factors
nR      = length(reac);
ndata   = time/tresol;
timings = [0,time,ndata];

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Estimate variances
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

varest;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Calculate SIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

senseit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Write SI results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

writeit;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Save workspace
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

save('..\results\workspace\nrosa.mat');