% OHA-based sensitivity indices (NROSA-OHA)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%  Noise Reduced Reaction-Oriented Sensitivity Analysis  %%%%%%%%%
%%%%%%%%%%                 OHA COMPUTATION                        %%%%%%%%%
%%%%%%%%%%             Authors: Zhang/Goutsias                    %%%%%%%%%
%%%%%%%%%%            Last Modified: 09/17/2010                   %%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% Calculate TESIs 

taut = UjT./VT;
taud = UjD./VD;
taus = UjS./VS;
taur = UjR./VR;

% Sort TESI values

[staut,IT] = sort(taut);
[staud,ID] = sort(taud);
[staus,IS] = sort(taus);
[staur,IR] = sort(taur);

% Determine U_0

epst = 0;
epsd = 0;
epss = 0;
epsr = 0;

tauttmp = 0;
taudtmp = 0;
taustmp = 0;
taurtmp = 0;

for m = 1:M
    taut0 = tauttmp;
    tauttmp = sum(staut(1:m));
    for q = 1:(m-1)
        for qq = (q+1):m
            tauttmp = tauttmp - VqqT(IT(q),IT(qq));
        end
    end
    if tauttmp > eps
        break;
    end
end
if m > 1
    epst = staut(m-1);
end

for m = 1:M
    taud0 = taudtmp;
    taudtmp = sum(staud(1:m));
    for q = 1:(m-1)
        for qq = (q+1):m
            taudtmp = taudtmp - VqqD(ID(q),ID(qq));
        end
    end
    if taudtmp > eps
        break;
    end
end
if m > 1
    epsd = staud(m-1);
end

for m = 1:M
    taus0 = taustmp;
    taustmp = sum(staus(1:m));
    for q = 1:(m-1)
        for qq = (q+1):m
            taustmp = taustmp - VqqS(IS(q),IS(qq));
        end
    end
    if taustmp > eps
        break;
    end
end
if m > 1
    epss = staus(m-1);
end

for m = 1:M
    taur0 = taurtmp;
    taurtmp = sum(staur(1:m));
    for q = 1:(m-1)
        for qq = (q+1):m
            taurtmp = taurtmp - VqqR(IR(q),IR(qq));
        end
    end
    if taurtmp > eps
        break;
    end
end
if m > 1
    epsr = staur(m-1);
end