% Response features (NSOSA-MC)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%% Noise Reduced Reaction-Oriented Sensitivity Analysis %%%%%%%%%%
%%%%%%%%%%%    MC COMPUTATION WITH DIMENSIONALITY REDUCTION      %%%%%%%%%%
%%%%%%%%%%%              Authors: Zhang/Goutsias                 %%%%%%%%%%
%%%%%%%%%%%             Last Modified: 09/20/2010                %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

function[TIM,DUR,STR,RAT] = featureit_EGFR(x,t,tresol,durtol,odetol)

% TIM:     ralaxation time
% DUR:     peak-sustained duration
% STR:     strength of sustained level
% RAT:     ratio of peak-sustained level
% species: molecular species of interest (1,2,...,N)
% x:       molecular activities

response = 2*(x(:,5) + x(:,7) + x(:,8) + x(:,11) + x(:,13) + x(:,16) + ...
    x(:,17) + x(:,19) + x(:,21));
tmax     = length(t);

STR = response(tmax);

norm_response = abs(response - STR);
trans_locat = find(norm_response >= (durtol*STR));
if length(trans_locat) > 1
    TIM = tresol*(max(trans_locat)-1);
else
    TIM = 0;
end

[peak,peak_time] = max(response);
DUR = TIM - tresol*(peak_time-1);

if STR > 0
    RAT = peak/STR;
else
    RAT = peak/odetol;
end

TIM = log(1+(TIM-1)*(TIM>0)); 
DUR = log(1+(DUR-1)*(DUR>0));
STR = log(1+(STR-1)*(STR>0));
RAT = log(1+(RAT-1)*(RAT>0));