% OHA-based sensitivity indices (NSOSA-OHA)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Noise Reduced Species-Oriented Sensitivity Analysis %%%%%%%%%%
%%%%%%%%%%%%                OHA COMPUTATION                      %%%%%%%%%%
%%%%%%%%%%%%            Authors: Zhang/Goutsias                  %%%%%%%%%%
%%%%%%%%%%%%           Last Modified: 09/20/2010                 %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

% Calculate TESIs 

taut = UjT./VT;
taud = UjD./VD;
taus = UjS./VS;
taur = UjR./VR;

% Sort TESI values

[staut,IT] = sort(taut);
[staud,ID] = sort(taud);
[staus,IS] = sort(taus);
[staur,IR] = sort(taur);

% Determine U_0

epst = 0;
epsd = 0;
epss = 0;
epsr = 0;

tauttmp = 0;
taudtmp = 0;
taustmp = 0;
taurtmp = 0;

for n = 1:N
    taut0 = tauttmp;
    tauttmp = sum(staut(1:n));
    for q = 1:(n-1)
        for qq = (q+1):n
            tauttmp = tauttmp - VqqT(IT(q),IT(qq));
        end
    end
    if tauttmp > eps
        break;
    end
end
if n > 1
    epst = staut(n-1);
end

for n = 1:N
    taud0 = taudtmp;
    taudtmp = sum(staud(1:n));
    for q = 1:(n-1)
        for qq = (q+1):n
            taudtmp = taudtmp - VqqD(ID(q),ID(qq));
        end
    end
    if taudtmp > eps
        break;
    end
end
if n > 1
    epsd = staud(n-1);
end

for n = 1:N
    taus0 = taustmp;
    taustmp = sum(staus(1:n));
    for q = 1:(n-1)
        for qq = (q+1):n
            taustmp = taustmp - VqqS(IS(q),IS(qq));
        end
    end
    if taustmp > eps
        break;
    end
end
if n > 1
    epss = staus(n-1);
end

for n = 1:N
    taur0 = taurtmp;
    taurtmp = sum(staur(1:n));
    for q = 1:(n-1)
        for qq = (q+1):n
            taurtmp = taurtmp - VqqR(IR(q),IR(qq));
        end
    end
    if taurtmp > eps
        break;
    end
end
if n > 1
    epsr = staur(n-1);
end