% Derivative-Based NSESI approximation (NSOSA-DA)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Noise Reduced Species-Oriented Sensitivity Analysis %%%%%%%%%%
%%%%%%%%%%%%         Derivative-Based Approximation              %%%%%%%%%%
%%%%%%%%%%%%            Authors: Zhang/Goutsias                  %%%%%%%%%%
%%%%%%%%%%%%           Last Modified: 09/20/2010                 %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Initializations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
sigmat(1:N) = 0;
sigmad(1:N) = 0;
sigmas(1:N) = 0;
sigmar(1:N) = 0;

vstoich = (-stoich.*(stoich < 0))';

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% MC-LHS iterations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Determine LHS permutations
%

for p = 1:P
    perm(p,:) = randperm(L);
end

for iter = 1:L
    
    iterleft = L+1-iter;
    fprintf('Remaining MC-LHS iterations: %6.0f\n',iterleft)
    
    %
    % Calculate Latin hypercube samples
    %
    
    uniform = rand(P,1);
    index   = (perm(:,iter)-1+uniform)./L;
    Z       = repmat(0,[P,1]);
    Z       = sigma*(-sqrt(2)*erfcinv(2*index)); % inverse normal CDF
    vzs     = V*Z;
    kappan  = kappa.*exp(vzs);
    
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Solve system of ODEs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

    xf = repmat(0,[tmax,N,M]);
    xb = repmat(0,[tmax,N,M]);
    for n = 1:N
    
        YS      = zeros(N,1);
        YS(n)   = delta;
        svyn    = vstoich*YS;
        % Michaelis-Menten kinetics
        svyn(7) = 0;
        svyn(8) = -YS(5);
        svyn(15) = 0;
        svyn(16) = -YS(9);
        svyn(31) = 0;
        svyn(32) = -YS(18);
        kappaf  = kappan.*exp(svyn);
        YS      = zeros(N,1);
        YS(n)   = -delta;
        svyn    = vstoich*YS;
        % Michaelis-Menten kinetics
        svyn(7) = 0;
        svyn(8) = -YS(5);
        svyn(15) = 0;
        svyn(16) = -YS(9);
        svyn(31) = 0;
        svyn(32) = -YS(18);
        kappab  = kappan.*exp(svyn);
    
        if iode == 1
            [t,xf(:,:,n)] = egfr(kappaf,x0,[odetol,odetol],timings);            
            [t,xb(:,:,n)] = egfr(kappab,x0,[odetol,odetol],timings);
        else
            [t,xf(:,:,n)] = ode15s(@netode_EGFR,tpoints,x0,options,kappaf,stoich);
            [t,xb(:,:,n)] = ode15s(@netode_EGFR,tpoints,x0,options,kappab,stoich);
        end
    
    end

    %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%% Calculate response characteristics
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %

    for n = 1:N
	    [TTf(n),DDf(n),SSf(n),RRf(n)] = featureit_EGFR(xf(:,:,n),t,tresol,durtol,odetol);
        [TTb(n),DDb(n),SSb(n),RRb(n)] = featureit_EGFR(xb(:,:,n),t,tresol,durtol,odetol);
    end
    
    sigmat = sigmat + ((TTf-TTb)./(2*delta)).^2;
    sigmad = sigmad + ((DDf-DDb)./(2*delta)).^2;
    sigmas = sigmas + ((SSf-SSb)./(2*delta)).^2;
    sigmar = sigmar + ((RRf-RRb)./(2*delta)).^2;
   
end    

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Approximate GSESIs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

sigmat = sigmat./sum(sigmat);
sigmad = sigmad./sum(sigmad);
sigmas = sigmas./sum(sigmas);
sigmar = sigmar./sum(sigmar); 

% Sort SESIs

[setat,IT] = sort(sigmat);
[setad,ID] = sort(sigmad);
[setas,IS] = sort(sigmas);
[setar,IR] = sort(sigmar);