#include <stdio.h>
#include <iostream>
#include <fstream>
#include "nr3matlab.h"
#include "ludcmp.h"
#include "stepper.h"
#include "odeint.h"
#include "steppersie.h"
#include "EGFReqns.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]){
	/*input is kappa vector, initial value vector, tolerances, and timings 
     * outputs are t vector and x vectors*/
    
	// usage:
	//
    // tol     = [tolAbs,tolRel]; // integration tolerances
	// timings = [tstart,tstop,numOutputPoints]; // times in sec
	// [t,x]   = egfr(kappa,x0,tol,timings);
	
	//set constants of integration
	const Int nvar=23; // number of species
	const Doub h1=.01, hmin=0.0;
    
	
	//Initialize matlab input output vars
	VecDoub kappa(prhs[0]);
	VecDoub xstart(prhs[1]);
	VecDoub tolerance(prhs[2]);
	VecDoub timings(prhs[3]);
	VecDoub t((timings[2]+1),plhs[0]);
    MatDoub x(nvar,(timings[2]+1),plhs[1]);
    

	//initialize ODE parameters
	Doub k1,k2,k3,k4,k5,k6,k7,k8,k9,k10,k11,k12,k13,k14,k15,k16,k17,k18,\
            k19,k20,k21,k22,k23,k24,k25,k26,k27,k28,k29,k30,k31,k32,k33,\
            k34,k35,k36,k37,k38,k39,k40,k41,k42,k43,k44,k45,k46,k47,k48,k49,k50; 
	
	k1=kappa[0];
	k2=kappa[1];
	k3=kappa[2];
	k4=kappa[3];
	k5=kappa[4];
	k6=kappa[5];
	k7=kappa[6];
	k8=kappa[7];
	k9=kappa[8];
	k10=kappa[9];
	k11=kappa[10];
	k12=kappa[11];
	k13=kappa[12];
	k14=kappa[13];
	k15=kappa[14];
	k16=kappa[15];
	k17=kappa[16];
	k18=kappa[17];
	k19=kappa[18];
	k20=kappa[19];
	k21=kappa[20];
	k22=kappa[21];
	k23=kappa[22];
	k24=kappa[23];
	k25=kappa[24];
	k26=kappa[25];
	k27=kappa[26];
	k28=kappa[27];
	k29=kappa[28];
	k30=kappa[29];
	k31=kappa[30];
	k32=kappa[31];
	k33=kappa[32];
	k34=kappa[33];
	k35=kappa[34];
	k36=kappa[35];
	k37=kappa[36];
	k38=kappa[37];
	k39=kappa[38];
	k40=kappa[39];
	k41=kappa[40];
	k42=kappa[41];
    k43=kappa[42];
	k44=kappa[43];
	k45=kappa[44];
	k46=kappa[45];
	k47=kappa[46];
	k48=kappa[47];
	k49=kappa[48];
	k50=kappa[49];

	
	Doub tstart, tstop;
	tstart = timings[0];
	tstop  = timings[1];
    
    // this makes sure the input variables are not modified in matlab
    VecDoub xstartTemp = xstart;
	
	Output out(timings[2]);
	
	
    // initialize rhs of ODE system
	rhs d(k1,k2,k3,k4,k5,k6,k7,k8,k9,k10,k11,k12,k13,k14,k15,k16,k17,k18,\
            k19,k20,k21,k22,k23,k24,k25,k26,k27,k28,k29,k30,k31,k32,k33,\
            k34,k35,k36,k37,k38,k39,k40,k41,k42,k43,k44,k45,k46,k47,k48,k49,k50);
    
    // create ODE integrator object
	Odeint<StepperSie<rhs> > ode(xstartTemp,tstart,tstop,tolerance[0],tolerance[1],h1,hmin,out,d);
	
    // perform integration
    ode.integrate();
    
    // write output to variables in matlab workspace
	for (Int i=0;i<out.count;i++){
		t[i]=out.xsave[i];
        for (Int j=0;j<x.nrows();j++){
            x[j][i]=out.ysave[j][i];
        }
	}
}
